"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ASTProvider = void 0;
const fs_1 = require("fs");
const vscode_uri_1 = require("vscode-uri");
const elmWorkspaceMatcher_1 = require("../util/elmWorkspaceMatcher");
class ASTProvider {
    constructor(connection, elmWorkspaces, documentEvents, parser) {
        this.connection = connection;
        this.parser = parser;
        this.handleChangeTextDocument = (params, elmWorkspace) => {
            this.connection.console.info(`Changed text document, going to parse it. ${params.textDocument.uri}`);
            const forest = elmWorkspace.getForest();
            const imports = elmWorkspace.getImports();
            const document = params.textDocument;
            let tree = forest.getTree(document.uri);
            if (tree === undefined) {
                const fileContent = fs_1.readFileSync(vscode_uri_1.URI.parse(document.uri).fsPath, "utf8");
                tree = this.parser.parse(fileContent);
            }
            for (const changeEvent of params.contentChanges) {
                tree = this.parser.parse(changeEvent.text);
            }
            if (tree) {
                forest.setTree(document.uri, true, true, tree, true);
                // Figure out if we have files importing our changed file - update them
                const urisToRefresh = [];
                for (const uri in imports.imports) {
                    if (imports.imports.hasOwnProperty(uri)) {
                        const fileImports = imports.imports[uri];
                        if (fileImports.some((a) => a.fromUri === document.uri)) {
                            urisToRefresh.push(uri);
                        }
                    }
                }
                urisToRefresh.forEach((a) => {
                    imports.updateImports(a, forest.getTree(a), forest);
                });
                // Refresh imports of the calling file
                imports.updateImports(document.uri, tree, forest);
            }
        };
        documentEvents.on("change", new elmWorkspaceMatcher_1.ElmWorkspaceMatcher(elmWorkspaces, (params) => vscode_uri_1.URI.parse(params.textDocument.uri)).handlerForWorkspace(this.handleChangeTextDocument));
    }
}
exports.ASTProvider = ASTProvider;
//# sourceMappingURL=astProvider.js.map