import { CodeAction, CodeActionParams, Diagnostic, ExecuteCommandParams, IConnection, ApplyWorkspaceEditResponse } from "vscode-languageserver";
import { TextDocument } from "vscode-languageserver-textdocument";
import { URI } from "vscode-uri";
import { IElmWorkspace } from "../../elmWorkspace";
import { Settings } from "../../util/settings";
import { TextDocumentEvents } from "../../util/textDocumentEvents";
import { DocumentFormattingProvider } from "../documentFormatingProvider";
export declare const CODE_ACTION_ELM_ANALYSE: string;
export declare const CODE_ACTION_ELM_ANALYSE_FIX_ALL: string;
export interface IElmAnalyseEvents {
    on(event: "new-report", diagnostics: Map<string, Diagnostic[]>): this;
}
export declare class ElmAnalyseDiagnostics {
    private connection;
    private events;
    private settings;
    private formattingProvider;
    private elmAnalysers;
    private diagnostics;
    private filesWithDiagnostics;
    private eventEmitter;
    private elmWorkspaceMatcher;
    constructor(connection: IConnection, elmWorkspaces: IElmWorkspace[], events: TextDocumentEvents<TextDocument>, settings: Settings, formattingProvider: DocumentFormattingProvider);
    on(event: string | symbol, listener: (...args: any[]) => void): this;
    updateFile(uri: URI, text?: string): Promise<void>;
    onCodeAction(params: CodeActionParams): CodeAction[];
    onExecuteCommand(params: ExecuteCommandParams): Promise<ApplyWorkspaceEditResponse | undefined>;
    private fixableDiagnostics;
    /**
     * If a diagnosticId is provided it will fix the single issue, if no
     * id is provided it will fix the entire file.
     */
    private fixer;
    private getFixEdits;
    private createEdits;
    private setupElmAnalyse;
    private onNewReportForWorkspace;
    private isFixable;
    private messageToDiagnostic;
}
