"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExposeUnexposeHandler = void 0;
const elmWorkspaceMatcher_1 = require("../../util/elmWorkspaceMatcher");
const vscode_uri_1 = require("vscode-uri");
const protocol_1 = require("../../protocol");
const refactorEditUtils_1 = require("../../util/refactorEditUtils");
class ExposeUnexposeHandler {
    constructor(connection, elmWorkspaces) {
        this.connection = connection;
        this.connection.onRequest(protocol_1.ExposeRequest, new elmWorkspaceMatcher_1.ElmWorkspaceMatcher(elmWorkspaces, (params) => vscode_uri_1.URI.parse(params.uri)).handlerForWorkspace(this.handleExposeRequest.bind(this)));
        this.connection.onRequest(protocol_1.UnexposeRequest, new elmWorkspaceMatcher_1.ElmWorkspaceMatcher(elmWorkspaces, (params) => vscode_uri_1.URI.parse(params.uri)).handlerForWorkspace(this.handleUnexposeRequest.bind(this)));
    }
    handleExposeRequest(params, elmWorkspace) {
        return __awaiter(this, void 0, void 0, function* () {
            const forest = elmWorkspace.getForest();
            const tree = forest.getTree(params.uri);
            if (tree) {
                const edits = refactorEditUtils_1.RefactorEditUtils.exposeValueInModule(tree, params.name);
                if (edits) {
                    yield this.connection.workspace.applyEdit({
                        changes: {
                            [params.uri]: [edits],
                        },
                    });
                }
            }
        });
    }
    handleUnexposeRequest(params, elmWorkspace) {
        return __awaiter(this, void 0, void 0, function* () {
            const forest = elmWorkspace.getForest();
            const tree = forest.getTree(params.uri);
            if (tree) {
                const edits = refactorEditUtils_1.RefactorEditUtils.unexposedValueInModule(tree, params.name);
                if (edits) {
                    yield this.connection.workspace.applyEdit({
                        changes: {
                            [params.uri]: [edits],
                        },
                    });
                }
            }
        });
    }
}
exports.ExposeUnexposeHandler = ExposeUnexposeHandler;
//# sourceMappingURL=exposeUnexposeHandler.js.map